
struct EditorView
    larvaviewer::ViewerView
    larvafilter::LarvaFilter
    tagfilter::TagFilter
    metadataeditor::MetadataEditor
    filemenu::FileMenu
    backendmenu::BackendMenu
    loadanimation::LoadAnimation
end

function JSServe.jsrender(session::Session, ev::EditorView)
    r(session, LarvaTaggerJS)
    r(session, LarvaTaggerCSS)
    r(session, LoadAwesomeCSS)
    r(session, JSServe.TailwindCSS)
    evaljs(session, js"window.addEventListener('beforeunload', LarvaTagger.confirmDialog)")
    r(session,
      DOM.div(ev.larvaviewer,
              cp(ControlPanel(ev.tagfilter; id="tag-panel", title="Tags"),
                 ControlPanel(ev.larvafilter; id="larva-panel", title="Tracks"),
                 ControlPanel(ev.filemenu, ev.metadataeditor, ev.backendmenu;
                              id="file-panel", title="Files", active=true)),
              ev.loadanimation;
              class="flex flex-row"))
end

projectdir = dirname(Base.active_project())

function larvaeditor(path=nothing; allow_multiple_tags::Union{Nothing, Bool}=nothing,
        backend_directory::String=projectdir)

    # to (re-)load a file, the app is reloaded with the filepath as sole information
    # from previous session
    input = Ref{Union{Nothing, String}}(path)

    App() do session::Session

        controller = ControllerHub()
        # used for method dispatch (here `Session` is taken to represent JSServe)
        controller[:frontend] = Session

        tryopenfile(controller, input)

        editor = EditorView(larvaviewer(controller;
                                        editabletags=true,
                                        multipletags=allow_multiple_tags),
                            larvafilter(controller),
                            tagfilter(controller),
                            metadataeditor(controller),
                            filemenu(controller),
                            backendmenu(controller, backend_directory),
                            loadanimation(controller))

        dom = JSServe.jsrender(session, editor)

        turn_load_animation_off(controller)

        return dom

    end
end
