
struct ViewerView
    assayplot::AssayViewer
    trackplot::TrackViewer
end

function JSServe.jsrender(session::Session, vv::ViewerView)
    controller = vv.trackplot.plot.controller
    delayed_controller = vv.assayplot.plot.controller

    assaydom = r(session, vv.assayplot)
    trackdom = r(session, vv.trackplot)

    on(larvaevents(controller).activated) do _
        vv.assayplot.visible[] = false
        JSServe.evaljs(session,
                       js"""
                       const assayviewer = $(assaydom);
                       const trackviewer = $(trackdom);
                       assayviewer.style.display = "none";
                       trackviewer.style.display = "block";
                       """)
    end
    on(larvaevents(controller).deactivated) do _
        vv.assayplot.visible[] = true
        JSServe.evaljs(session,
                       js"""
                       const assayviewer = $(assaydom);
                       const trackviewer = $(trackdom);
                       trackviewer.style.display = "none";
                       assayviewer.style.display = "block";
                       """)
    end

    on(session.on_close) do closed
        if closed
            stop!(delayed_controller)
        end
    end

    r(session,
      DOM.div(JSServe.TailwindCSS, assaydom, trackdom; class="flex flex-row"))
end

function larvaviewer(path::String; allow_multiple_tags::Union{Nothing, Bool}=false)

    App() do session::Session

        controller = ControllerHub()

        tryopenfile(controller, path)

        viewer = larvaviewer(controller; multipletags=allow_multiple_tags)

        r(session, LarvaTaggerJS)
        r(session, LarvaTaggerCSS)

        JSServe.jsrender(session, viewer)
    end
end

function larvaviewer(controller;
        editabletags::Bool=false,
        multipletags::Union{Nothing, Bool}=false,
    )
    model = getlarvae(controller)
    times = gettimes(controller)
    tag_lut = gettags(controller)

    controller[:player] = player = timecontroller(times)
    controller[:larva] = larva = LarvaController(controller, model, tag_lut, player)

    n_simultaneous_larvae = simultaneouslarvae(model)
    if 100 < n_simultaneous_larvae
        @info "Refresh rate throttled to 1Hz" n_simultaneous_larvae
        delayed_controller = slave(larva, Cooldown(1.0))
    elseif 50 < n_simultaneous_larvae
        @info "Refresh rate throttled to 2Hz" n_simultaneous_larvae
        delayed_controller = slave(larva, Cooldown(0.5))
    else
        delayed_controller = larva
    end

    on(getactivelarva(controller)) do id
        @info isnothing(id) ? "No active larva" : "Activating larva #$(id)"
    end

    viewer = ViewerView(assayviewer(delayed_controller),
                        trackviewer(controller;
                                    editabletags=editabletags,
                                    multipletags=multipletags))

    settimestep!(controller, 1)

    return viewer
end
